using BPA.SAAS.Manage.Application.AExternalPlatform.BaseDto;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.Bom.Dtos;
using BPA.SAAS.Manage.Application.DataBase.Dtos.Bom;
using BPA.SAAS.Manage.Core.Base;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.AExternalPlatform.Service.Bom.Services
{
    public interface IBomService
    {
        /// <summary>
        /// 分页查询
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        Task<PageUtil<List<BomDto>>> GetBomPageList(BomPageQueryDto inputDto);
        /// <summary>
        /// 添加
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        Task<bool> AddBom(BaseRequestDto<BomInsertDto> dto);
        /// <summary>
        /// 更新
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        Task<bool> UpdateBom(BaseRequestDto<BomUpdateDto> dto);
        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="ids"></param>
        /// <returns></returns>
        Task<bool> DeleteBom(DeleteBomDto dto);
        /// <summary>
        /// 根据配方id查询配方详情
        /// </summary>
        /// <param name="bomId"></param>
        /// <returns></returns>
        Task<List<BomEntryDto>> GetBomEntry(BomIdDto dto);
        /// <summary>
        /// 添加配方详情
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        Task<bool> AddBomEntry(BomEntryInsertDto dto);
    }
}
